
# coding: utf-8


# # Data analysis with Python: maritime climate report
#
# ### Dept. Mecánica de Estructuras e Ingeniería Hidráulica
# ### E.T.S.I. Caminos, Canales y Puertos
# ### Universidad de Granada

# ## Download files for the practice
# https://gdfa.ugr.es/share/pedro/curso_etsiccp.zip

# ## Block I - Read data

# Open the file `SIMAR_tutorial.txt` and check the following items:
#
# 1\. Has it header? How many lines?
#
# 2\. How are the columns delimited (white space, tabulation, comma, etc)?
#
# 3\. Are there null values?
#
# 4\. Are there columns with dates?

# ## Activity I.1
# Read the `SIMAR_tutorial.txt` file using the function **read_csv** inside the **pandas** module.


import pandas as pd

simar_tutorial = pd.read_csv('SIMAR_tutorial.txt')
print(simar_tutorial)


# Eliminar cabecera con el parámetro `skiprows`=n_filas cabecera


# Elimino cabecera
simar_tutorial = pd.read_csv('SIMAR_tutorial.txt',
                             skiprows=10)
print(simar_tutorial)


# Añadir delimitador de columnas con el parámetro `delim_whitespace`=True


# Añado delimitador
simar_tutorial = pd.read_csv('SIMAR_tutorial.txt',
                             skiprows=10,
                             delim_whitespace=True)
print(simar_tutorial)


# Eliminar valores nulos con el parámetro `na_values`=tipo de dato nulo


# Elimino valores nulos
simar_tutorial = pd.read_csv('SIMAR_tutorial.txt',
                             skiprows=10,
                             delim_whitespace=True,
                             na_values=-9999)
print(simar_tutorial)


# Leo las columnas de fecha como dato tipo fecha (DateTime)


# Leo las columnas de fecha como dato tipo fecha (DateTime)
simar_tutorial = pd.read_csv('SIMAR_tutorial.txt',
                             skiprows=10,
                             delim_whitespace=True,
                             na_values=-9999,
                             parse_dates=[[0, 1]],
                             index_col=0)
print(simar_tutorial)


# ## Exercise I.1
# Read the SIMAR_1052046.txt file
#


simar = pd.read_table('SIMAR_1052046.txt',
                     skiprows=81,
                     delim_whitespace=True,
                     na_values=-99.9,
                     parse_dates=[[0, 1, 2, 3]],
                     index_col=0)

print(simar)


# ## Activity I.2
# Represent the location of a point in a map


import smopy
import matplotlib.pyplot as plt

#%% Bloque II Mapa
# Create de figure
figure = plt.figure(figsize=(16, 12))
ax = plt.axes()
plt.xticks([])
plt.yticks([])

#Plot the map and the point
map = smopy.Map((45., 0., 50., 5.), z=5)
x, y = map.to_pixels(48, 2.3)
map.show_mpl(ax)
ax.plot(x, y, 'ok', ms=20)

# Show figure
plt.show()

#Save figure (optional)
figure.savefig('point_location.png')


# ## Exercise I.2
# Represent the location of the SIMAR_1052046 data


get_ipython().magic(u'matplotlib inline')
import smopy
import matplotlib.pyplot as plt

figure = plt.figure(figsize=(16, 12))
ax = plt.axes()
plt.xticks([])
plt.yticks([])

map = smopy.Map((36., -7., 37., -5.), z=5)
x, y = map.to_pixels(36.5, -6.5)
map.show_mpl(ax)

ax.plot(x, y, '^r', ms=20)

figure.savefig('simar_cadiz_location.png')


# ## Block II - Quick summary of the data

# ## Activity II.1
# What is the maximum registered significant wave height (Hm0)?


Hm0_max = np.max(simar.loc[:, 'Hm0'])
print(Hm0_max)


# What is the more frequent direction of origin of the wind (DirV)?


from scipy import stats

wind_dir = stats.mode(simar.loc[:, 'DirV'])
print(wind_dir)


# ## Exercise II.1
# Calculate the following values of the SIMAR data set:
# - **EASY** The maximum registered value of Wind Velocity (VelV)
# - **EASY** The average direction of origin of the wave (DirM) using the function **np.mean()**
# - **MEDIUM** The minum registered peak period (Tp) between 2001 and 2005
# - **HARD** The instants where the significant wave height (Hm0) is higher than 3 meters in 2017. (Hint: Use the function **np.where()**)
#

# 1. The maximum registered value of Wind Velocity (VelV)
#


max_wind_vel = np.max(simar.loc[:, 'VelV'])
print(max_wind_vel)


# 2. The average direction of origin of the wave (DirM)
#


average_dirM = np.mean(simar.loc[:, 'DirM'])
print(average_dirM)


# 3. The minum registered peak period (Tp) between 2001 and 2005


min_tp = np.min(simar.loc['2001':'2005', 'Tp'])
print(min_tp)


# 4. The instants where the significant wave height (Hm0) is higher than 3 meters in 2017


# Crop the simar
simar_crop = simar.loc['2017', 'Hm0']
print(simar_crop)



# Obtain the positions that match the condition
[pos, ] = np.where(simar_crop > 5)
print(pos)



# Evaluate the positions
instants = simar_crop[pos]
print(instants)


# ## Activity II.2
# Describe function of pandas


table = simar.describe()
print(table)


# ## Block III - Plotting data for analysis

# ## Activity III.1
# Plot time series of `Hm0` using pandas (**quick** but **not customizable** graphics)


simar.loc[:, 'Hm0'].plot()


# ## Activity III.2
# Plotting time series using matplotlib


fig = plt.figure(figsize=(10, 5))
ax = plt.axes()
plt.show()



fig = plt.figure(figsize=(10, 5))
ax = plt.axes()


ax.plot(simar.loc[:, 'Hm0'])
plt.show()



fig5 = plt.figure(figsize=(10, 5))
ax = plt.axes()

ax.plot(simar.loc[:, 'Hm0'])

ax.set_xlabel('Time (years)', fontsize=14)
ax.set_ylabel('Hm0 (m)', fontsize=14)
plt.title('Significant wave height', fontsize=18)
ax.grid()
plt.show()
fig5.savefig('Hm0_plot.jpg')


# ## Exercise III.2
# Plot time series of another variable between 2005 and 2010


simar.loc['2005':'2010', 'VelV'].plot()



fig = plt.figure(figsize=(10, 5))
ax = plt.axes()

ax.plot(simar.loc['2005':'2010', 'VelV'])
ax.set_xlabel('Time (years)', fontsize=14)
ax.set_ylabel('W_v (m/s)', fontsize=14)
plt.title('Wind velocity', fontsize=18)
ax.grid()


# ## Activity III.3
# Plot histogram of `DirM` using Pandas


simar.loc[:, 'DirM'].plot.hist(bins=10)


# ## Exercise III.4
# Compare the histograms of `Hm0` between january and august of a selected year (2015) using Pandas


# Extract
indices = np.where(simar.index.year == 2015)
print(indices)



simar_2015 = simar.iloc[indices[0], :]
print(simar_2015)



# Extract month
indices = np.where(simar_2015.index.month == 1)
simar_2015_01 = simar_2015.iloc[indices[0], :]
print(simar_2015_01)



# Extract Hm0
simar_2015_01_Hm0 = simar_2015_01.loc[:, 'Hm0']
print(simar_2015_01_Hm0)



# Repeat for the the month of august
indices = np.where(simar.index.year == 2015)
simar_2015 = simar.iloc[indices[0], :]
indices = np.where(simar_2015.index.month == 8)
simar_2015_8 = simar_2015.iloc[indices[0],:]
simar_2015_8_Hm0 = simar_2015_8.loc[:, 'Hm0']
print(simar_2015_8_Hm0)



# An easier and faster way
simar_2015_1_Hm0 = simar['2015-01']['Hm0']
simar_2015_8_Hm0 = simar['2015-08']['Hm0']



# Plot subplot figure
plt.subplot(1, 2, 1)
simar_2015_1_Hm0.plot.hist(bins=10)
plt.xlabel('Hm0 (m)')
plt.ylabel('Frequency')
plt.title('Hm0 in january')
plt.grid(True)

plt.subplot(1, 2, 2)
simar_2015_8_Hm0.plot.hist(bins=10)
plt.xlabel('Hm0 (m)')
plt.ylabel('Frequency')
plt.title('Hm0 in august')
plt.grid(True)

plt.tight_layout()
plt.show()


# ## Activity III.4
# Plot histogram of `DirM` using Seaborn
#


import seaborn as sns

values = simar.loc[:, 'DirM']
ax = sns.distplot(values, bins = 20)


# ## Activity III.5
# Dispersion graphic using Pandas between `Hm0` and `Tp`


simar.plot.scatter('Hm0', 'Tp')


# ## Exercise III.5
# Compare and plot the dispersion graphic between `VelV` and `DirV` 1959 and 2016


# Extraigo el año
indices = np.where(simar.index.year == 1959)
simar_1959 = simar.iloc[indices[0], :]
print(simar_1959)



# Extraigo el descriptor VelV
simar_1959_velv = simar_1959.loc[:, 'VelV']
print(simar_1959_velv)



Velv_1959 = simar.loc['1959', 'VelV']
Dirv_1959 = simar.loc['1959', 'DirV']
Velv_2016 = simar.loc['2016', 'VelV']
Dirv_2016 = simar.loc['2016', 'DirV']



# Plot subplot figure
plt.subplot(1, 2, 1)
plt.plot(Velv_1959, Dirv_1959, '.')
plt.xlabel('VelV (m/s)')
plt.ylabel('DirV (deg)')
plt.title('VelV - DirV in 1959')
plt.grid(True)

plt.subplot(1, 2, 2)
plt.plot(Velv_2016, Dirv_2016, '.')
plt.xlabel('VelV (m/s)')
plt.ylabel('DirV (deg)')
plt.title('VelV - DirV in 2016')
plt.grid(True)

plt.tight_layout()
plt.show()


# ## Activity III.5
# Dispersion graphic using Seaborn between `Hm0` and `DirM`


ax = sns.jointplot('Hm0', 'DirM', simar)
ax.ax_joint.set_xlabel('Hm0 (m)', fontsize=14)
ax.ax_joint.set_ylabel('DirM (deg)', fontsize=14)


# ## Activity III.6
# Wave rose


from windrose import plot_windrose

ax = plot_windrose(simar, kind='bar', var_name='Hm0', direction_name='DirM', normed=True, opening=0.8)
ax.legend(loc='upper left', fontsize='small')


# ## Exercise III.6
# Wind rose


from windrose import plot_windrose

ax = plot_windrose(simar, kind='bar', var_name='VelV', direction_name='DirV', normed=True, opening=0.8)
ax.legend(loc='upper left', fontsize='small')


# ## Block IV - Find and fill data gaps

# Read the file `SIMAR_gaps.txt` using **read_csv**


#Read simar gaps
simar_gaps = pd.read_csv('SIMAR_gaps.txt',
                     skiprows=81,
                     delim_whitespace=True,
                     na_values=-99.9,
                     parse_dates=[[0, 1, 2, 3]],
                     index_col=0)
print(simar_gaps)



# Find time step
time = simar_gaps.index.to_series().diff()
time_step = stats.mode(time.values)
time_step = pd.to_timedelta(time_step[0])
print(time_step)



# Reindex to the actual timestep to change gaps for nan
new_index = pd.date_range(simar_gaps.index[0], simar.index[-1], freq=time_step[0])
simar_gaps_reindex = simar_gaps.reindex(new_index)



# Interpolate gaps
simar_fill = simar_gaps_reindex.interpolate(method='linear', axis=0, limit=25, limit_direction='both')
print(simar_fill)


# Represent the original and interpolated `Hm0` time series


fig = plt.figure(figsize=(10, 5))
ax = plt.axes()

ax.plot(simar_gaps.loc[:, 'Hm0'], '.')
ax.set_xlabel('Time (years)', fontsize=14)
ax.set_ylabel('Hm0 (m)', fontsize=14)
plt.title('Significant wave height', fontsize=18)
ax.grid()




fig = plt.figure(figsize=(10, 5))
ax = plt.axes()

ax.plot(simar_gaps.loc[:, 'Hm0'], '.')
ax.set_xlabel('Time (years)', fontsize=14)
ax.set_ylabel('Hm0 (m)', fontsize=14)
plt.title('Significant wave height', fontsize=18)
ax.grid()
ax.plot(simar_fill.loc[:, 'Hm0'])



simar_fill = simar_gaps_reindex.interpolate(method='linear', axis=0, limit=48, limit_direction='both')


# Represent the original and interpolated `Hm0` time series


fig = plt.figure(figsize=(10, 5))
ax = plt.axes()

ax.plot(simar_gaps.loc[:, 'Hm0'], '.')
ax.set_xlabel('Time (years)', fontsize=14)
ax.set_ylabel('Hm0 (m)', fontsize=14)
plt.title('Significant wave height', fontsize=18)
ax.grid()
ax.plot(simar_fill.loc[:, 'Hm0'])

